' Class for abstract logging to one or more of the following facilities:
'
' - interactive console
' - log file
' - eventlog
'
' Author:  Ansgar Wiechers <ansgar.wiechers@planetcobalt.net>
' Date:    2010-06-05
' Version: 1.0

Option Explicit

Const vbDebug = &h0050

Const INTERNAL_ERROR     = 51
Const ILLEGAL_ASSIGNMENT = 501

Class Logger
	Private validLogLevels

	' Property "LogToConsole" (Boolean, rw)
	'
	' If True, messages are logged to the console either via MsgBox() (when the
	' script is executed with wscript.exe) or via WriteLine() (when the script is
	' executed via cscript.exe).
	'
	' Enabled by default when the script is run interactively.
	Private logToConsoleEnabled

	Public Property Get LogToConsole
		LogToConsole = logToConsoleEnabled
	End Property

	Public Property Let LogToConsole(ByVal enable)
		logToConsoleEnabled = Not Not enable
	End Property

	' Property "LogToFile" (Boolean, ro)
	'
	' If True, messages are logged to a text file. Logging is enabled by setting
	' the property LogFile to a non-zero string.
	'
	' Disabled by default.
	Private logToFileEnabled

	Public Property Get LogToFile
		LogToFile = logToFileEnabled
	End Property

	' Property "LogFile" (String, ro)
	'
	' Name of the file that messages are logged to. Setting this property to a
	' non-empty string enables logging to a file with the given filename,
	' setting it to the empty string disables file logging.
	'
	' Default is the empty string (filename not set).
	Private logFileName
	Private logFileHandle

	Public Property Get LogFile
		LogFile = logFileName
	End Property

	Public Property Let LogFile(ByVal filename)
		Dim fso, ioMode

		filename = Trim(Replace(filename, vbTab, " "))
		If filename = "" Then
			' Close a previously opened log file.
			If Not logFileHandle Is Nothing Then
				logFileHandle.Close
				Set logFileHandle = Nothing
			End If
			logToFileEnabled = False
		Else
			Set fso = CreateObject("Scripting.FileSystemObject")
			filename = fso.GetAbsolutePathName(filename)
			If logFileName <> filename Then
				' Close a previously opened log file.
				If Not logFileHandle Is Nothing Then logFileHandle.Close

				If overwriteFile Then
					ioMode = 2  ' open for (over)writing
				Else
					ioMode = 8  ' open for appending
				End If

				' Open log file either as ASCII or Unicode, depending on system settings.
				Set logFileHandle = fso.OpenTextFile(filename, ioMode, -2)

				logToFileEnabled = True
			End If
			Set fso = Nothing
		End If

		logFileName = filename
	End Property

	' Property "Overwrite" (Boolean, rw)
	'
	' If True, an existing log file is overwritten, otherwise the messages will
	' be appended to an existing log file.
	' This property affects only logging to a file and is ignored otherwise.
	' Changes to this property will not affect already opened log files until
	' they're re-opened.
	'
	' Default is False (append rather than overwrite).
	Private overwriteFile

	Public Property Get Overwrite
		Overwrite = overwriteFile
	End Property

	Public Property Let Overwrite(ByVal enable)
		overwriteFile = Not Not enable
	End Property

	' Property "Separator" (String, rw)
	'
	' This property defines a separator character that is used to separate fields
	' in messages written to log files. The property is ignored by all other
	' facilities.
	'
	' Default is to separate fields with Tabs.
	Private sep

	Public Property Get Separator
		Separator = sep
	End Property

	Public Property Let Separator(ByVal char)
		If Len(char) <> 1 Then
			Err.Raise ILLEGAL_ASSIGNMENT, WScript.ScriptName, "Separator must be a single character."
		Else
			sep = char
		End If
	End Property

	' Property "LogToEventlog" (Boolean, rw)
	'
	' If True, messages are logged to the Application eventlog.
	'
	' Enabled by default when the script is run non-interactively.
	Private logToEventlogEnabled
	Private sh

	Public Property Get LogToEventlog
		LogToEventlog = logToEventlogEnabled
	End Property

	Public Property Let LogToEventlog(ByVal enable)
		logToEventlogEnabled = Not Not enable
		If sh Is Nothing And logToEventlogEnabled Then
			Set sh = CreateObject("WScript.Shell")
		ElseIf Not (sh Is Nothing Or logToEventlogEnabled) Then
			Set sh = Nothing
		End If
	End Property

	' Property "Timestamp" (Boolean, rw)
	'
	' If True, a timestamp is logged with each message. This option has no effect
	' on Eventlog logging, because events are always timestamped anyway.
	'
	' Disabled by default.
	Private addTimestamp

	Public Property Get Timestamp
		Timestamp = addTimestamp
	End Property

	Public Property Let Timestamp(ByVal enable)
		addTimestamp = Not Not enable
	End Property

	' Property "Debug" (Boolean, rw)
	'
	' If True, enable logging of debug messages. Otherwise debug messages are
	' silently discarded.
	'
	' Disabled by default.
	Private debugEnabled

	Public Property Get Debug
		Debug = debugEnabled
	End Property

	Public Property Let Debug(ByVal enable)
		debugEnabled = Not Not enable
	End Property

	' - Constructor/Destructor ---------------------------------------------------

	Private Sub Class_Initialize()
		logToConsoleEnabled = WScript.Interactive

		logToFileEnabled = False
		logFileName = ""
		Set logFileHandle = Nothing
		overwriteFile = False
		sep = vbTab

		logToEventlogEnabled = Not WScript.Interactive
		Set sh = Nothing

		addTimestamp = False
		debugEnabled = False

		Set validLogLevels = CreateObject("Scripting.Dictionary")
		validLogLevels.Add vbInformation, True
		validLogLevels.Add vbExclamation, True
		validLogLevels.Add vbCritical, True
		validLogLevels.Add vbDebug, True
	End Sub

	Private Sub Class_Terminate()
		If Not logFileHandle Is Nothing Then
			logFileHandle.Close
			Set logFileHandle = Nothing
			logFileName = ""
		End If

		Set sh = Nothing
	End Sub

	' ----------------------------------------------------------------------------

	' Convenient alias for LogInfo().
	Public Sub Log(msg)
		LogInfo msg
	End Sub

	' Log message with log level "information".
	Public Sub LogInfo(msg)
		LogMessage msg, vbInformation
	End Sub

	' Log message with log level "warning".
	Public Sub LogWarning(msg)
		LogMessage msg, vbExclamation
	End Sub

	' Log message with log level "error".
	Public Sub LogError(msg)
		LogMessage msg, vbCritical
	End Sub

	' Log message with log level "debug" only if debugging is enabled.
	Public Sub LogDebug(msg)
		If debugEnabled Then LogMessage msg, vbDebug
	End Sub

	' Log given message with given log level to all enabled facilities.
	Private Sub LogMessage(msg, logLevel)
		Dim tstamp, prefix

		If Not validLogLevels.Exists(logLevel) Then Err.Raise INTERNAL_ERROR, WScript.ScriptName, "Undefined logLevel '" & logLevel & "'."

		tstamp = Now
		prefix = ""

		' Log to facilite "Console". If the script is run with cscript.exe, messages
		' are printed to StdOut or StdErr, depending on log level. If the script is
		' run with wscript.exe, messages are displayed as MsgBox() pop-ups.
		If logToConsoleEnabled Then
			If InStr(LCase(WScript.FullName), "cscript") <> 0 Then
				If addTimestamp Then prefix = tstamp & vbTab
				Select Case logLevel
					Case vbDebug:       WScript.StdOut.WriteLine prefix & "DEBUG: " & msg
					Case vbInformation: WScript.StdOut.WriteLine prefix & msg
					Case vbExclamation: WScript.StdErr.WriteLine prefix & "Warning: " & msg
					Case vbCritical:    WScript.StdErr.WriteLine prefix & "Error: " & msg
				End Select
			Else
				If addTimestamp Then prefix = tstamp & vbNewLine & vbNewLine
				If logLevel = vbDebug Then
					MsgBox prefix & msg, vbOKOnly Or vbInformation, WScript.ScriptName & " (Debug)"
				Else
					MsgBox prefix & msg, vbOKOnly Or logLevel, WScript.ScriptName
				End If
			End If
		End If

		' Log to facility "Logfile".
		If logToFileEnabled Then
			If addTimestamp Then prefix = tstamp & sep
			Select Case logLevel
				Case vbDebug:       logFileHandle.WriteLine prefix & "DEBUG" & sep & msg
				Case vbInformation: logFileHandle.WriteLine prefix & "INFO" & sep & msg
				Case vbExclamation: logFileHandle.WriteLine prefix & "WARN" & sep & msg
				Case vbCritical:    logFileHandle.WriteLine prefix & "ERROR" & sep & msg
			End Select
		End If

		' Log to facility "Eventlog".
		' Timestamps are automatically logged with this facility, so addTimestamp
		' can be ignored.
		If logToEventlogEnabled Then
			Select Case logLevel
				Case vbDebug:       sh.LogEvent 4, "DEBUG: " & msg
				Case vbInformation: sh.LogEvent 4, msg
				Case vbExclamation: sh.LogEvent 2, msg
				Case vbError:       sh.LogEvent 1, msg
			End Select
		End If
	End Sub
End Class
